# -*- coding: utf-8 -*-
import logging
from functools import wraps

from auth import AccessAttempt

logger = logging.getLogger(__name__)

def watch_login(func):
    """
    Decorator @watch_login
    :param func:
    :return:
    """
    @wraps(func)
    def decorated_login(request, *args, **kwargs):

        response = func(request, *args, **kwargs)

        lockout = AccessAttempt(request, response)
        user_access = lockout.check_failed_login()

        if user_access.is_locked or lockout.is_ip_banned:
            return lockout.get_lockout_response()

        return response

    return decorated_login


def protect_login_with_captcha(func):
    """
    Protect the login with a captcha after of N failed access attempts
    :param func:
    :return:
    """
    @wraps(func)
    def decorated_login(request, *args, **kwargs):
        response = func(request, *args, **kwargs)

        access_attempt = AccessAttempt(request, response)
        access_attempt.inspect()

        return response

    return decorated_login


from django.conf.urls import patterns, include, url
from django.contrib.auth.views import login

from doberman.decorators import watch_login

urlpatterns = patterns('',
                       url(r'^login/$', watch_login(login), name='login', kwargs={'template_name': 'login.html'}),
                       url(r'^logout/$', 'django.contrib.auth.views.logout', name='logout', kwargs={'next_page': '/'}),
                       )


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('doberman', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='failedaccessattempt',
            name='captcha_attempts',
            field=models.SmallIntegerField(default=0),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='failedaccessattempt',
            name='captcha_enabled',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='failedaccessattempt',
            name='captcha_passed',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='failedaccessattempt',
            name='is_expired',
            field=models.BooleanField(default=False, verbose_name='Lock expired'),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='failedaccessattempt',
            name='is_locked',
            field=models.BooleanField(default=False, verbose_name='User/IP Locked'),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='failedaccessattempt',
            name='modified',
            field=models.DateTimeField(auto_now=True, verbose_name='Last attempt'),
            preserve_default=True,
        ),
    ]



from django.contrib import admin
from models import *


class FailedAccessAttemptAdmin(admin.ModelAdmin):
    """
    Failed Access Attempt Admin
    """
    list_display = (
        'modified',
        'is_locked',
        'is_expired',
        'username',
        'ip_address',
        'failed_attempts',
        'captcha_enabled',
        'captcha_passed',
        'captcha_attempts',
        'user_agent'
    )
    search_fields = ['username']

admin.site.register(FailedAccessAttempt, FailedAccessAttemptAdmin)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='FailedAccessAttempt',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('username', models.CharField(max_length=255, verbose_name='Username')),
                ('user_agent', models.CharField(max_length=255, verbose_name="The client's user agent string", blank=True)),
                ('ip_address', models.IPAddressField(verbose_name='The IP address of the client')),
                ('failed_attempts', models.PositiveIntegerField(default=0, verbose_name='Failed attempts')),
                ('is_locked', models.BooleanField(default=False)),
                ('is_expired', models.BooleanField(default=False)),
                ('params_post', models.TextField(verbose_name='GET data')),
                ('params_get', models.TextField(verbose_name='POST data')),
            ],
            options={
                'ordering': ('-created', 'username'),
                'abstract': False,
                'db_table': 'doberman_failed_access_attempt',
                'verbose_name': 'Failed access attempt',
                'verbose_name_plural': 'Failed access attempts',
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-
try:
    from django.utils import timezone
except ImportError:
    from datetime import datetime as timezone

from django.db import models
from django.utils.translation import ugettext_lazy as _

from settings import DOBERMAN_LOGIN_FORGOTTEN_SECONDS


class AbstractFailedAccessAttempt(models.Model):
    """
    Abstract Failed Access Attempt
    """
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True, verbose_name=_(u'Last attempt'))

    username = models.CharField(
        max_length=255,
        verbose_name=_("Username"),
    )
    user_agent = models.CharField(
        max_length=255, blank=True,
        verbose_name=_("The client's user agent string")
    )

    ip_address = models.IPAddressField(
        verbose_name=(_("The IP address of the client"))
    )

    failed_attempts = models.PositiveIntegerField(verbose_name=_(u'Failed login attempts'), default=0)
    is_locked = models.BooleanField(default=False, verbose_name=_(u'User/IP Locked'))
    is_expired = models.BooleanField(default=False, verbose_name=u'Lock expired')

    captcha_enabled = models.BooleanField(default=False, verbose_name=_("Captcha protection"))
    captcha_passed = models.BooleanField(default=False)
    captcha_attempts = models.SmallIntegerField(default=0, verbose_name=_("Captcha failed attempts"))

    class Meta:
        abstract = True
        ordering = ('-modified',)
        verbose_name = _("Access attempts")
        verbose_name_plural = _("Access attempts")

    def __str__(self):
        return self.username

    @classmethod
    def get_last_failed_access_attempt(cls, **kwargs):
        """
        Return Failed access attempt of Client
        :param ip_adress: String
        :return:
        """
        try:
            lockout = cls.objects.get(
                **kwargs
            )
        except cls.DoesNotExist:
            lockout = None

        if lockout:
            time_remaining = lockout.expiration_time
            if time_remaining and time_remaining <= 0:
                lockout.is_expired = True
                lockout.save()
                return None

        return lockout

    @property
    def expiration_time(self):
        """
        Returns the time until this access attempt is forgotten.
        """
        logging_forgotten_time = DOBERMAN_LOGIN_FORGOTTEN_SECONDS

        if logging_forgotten_time <= 0:
            return None

        now = timezone.now()
        delta = now - self.modified
        time_remaining = logging_forgotten_time - delta.seconds

        return time_remaining


class FailedAccessAttempt(AbstractFailedAccessAttempt):
    """
    Failed Access Attemps
    """
    params_post = models.TextField(verbose_name=_("GET data"))
    params_get = models.TextField(verbose_name=_("POST data"))


#!/usr/bin/env python
from setuptools import setup, find_packages

requirements = ['Django>=1.7.0', ]

try:
    from unittest import mock
except ImportError:
    requirements.append('mock')

setup(
    name="django-doberman",
    version="0.5.6",
    author="Nicolas Mendoza",
    author_email="niccolasmendoza@gmail.com",
    maintainer='Nicolas Mendoza',
    maintainer_email='niccolasmendoza@gmail.com',
    description="Django app that locks out users after too many failed login attempts.",
    long_description=open('README.rst').read(),
    license="MIT License",
    keywords="django locks users account login attempts banned ip doberman authentication",
    url="https://github.com/nicchub/django-doberman",
    packages=[
        'doberman'
    ],
    include_package_data=True,
    tests_require=['python-coveralls'],

    install_requires=requirements,

    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.2",
        "Framework :: Django",
        "Framework :: Django :: 1.7",
        "Framework :: Django :: 1.8",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries"
    ]
)


# -*- coding: utf-8 -*-
from django.core.handlers.wsgi import WSGIRequest
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.apps import apps

from contrib.ipware import AccessIPAddress
from exceptions import DobermanImproperlyConfigured
from settings import (
    DOBERMAN_USERNAME_FORM_FIELD,
    DOBERMAN_MAX_FAILED_ATTEMPTS,
    DOBERMAN_LOCKOUT_TIME,
    DOBERMAN_IPLOCKOUT_TEMPLATE,
    DOBERMAN_CUSTOM_MODEL
)

def get_doberman_model():
        try:
            return apps.get_model(DOBERMAN_CUSTOM_MODEL)
        except ValueError:
            raise DobermanImproperlyConfigured("AUTH_USER_MODEL must be of the form 'app_label.model_name'")
        except LookupError:
            raise DobermanImproperlyConfigured(
                "DOBERMAN-MODEL refers to model '%s' that has not been installed: " % DOBERMAN_CUSTOM_MODEL
            )

class AccessAttempt(AccessIPAddress):
    """
    Failed Access Attempt class
    """
    max_failed_attempts = DOBERMAN_MAX_FAILED_ATTEMPTS
    block_login_seconds = DOBERMAN_LOCKOUT_TIME
    template_name = DOBERMAN_IPLOCKOUT_TEMPLATE

    def __init__(self, request, response):
        super(AccessAttempt, self).__init__()

        if isinstance(request, WSGIRequest):
            self.request = request
        else:
            self.request = request.request  #cbv

        self.response = response

        self.ip = self.get_client_ip_address(self.request)

        self.last_attempt_instance = None
        self.username = self.request.POST.get(DOBERMAN_USERNAME_FORM_FIELD, None)

        self._FailedAccessAttemptModel = get_doberman_model()  # doberman supported custom models, see documentation

    def get_last_failed_access_attempt(self, **kwargs):
        """
        Return the last failed access attempt or None,
        the model can be change but is obligatory implement the method "get_last_failed_access_attempt"
        """

        last_failed_access = self._FailedAccessAttemptModel.get_last_failed_access_attempt(
            **kwargs
        )

        return last_failed_access

    def check_failed_login(self):
        """
        'Private method', check failed logins, it's used for wath_login decorator
        """
        last_attempt = self.get_last_failed_access_attempt()

        if not last_attempt:
            # create a new entry
            user_access = self._FailedAccessAttemptModel(ip_address=self.ip)
        elif last_attempt:
            user_access = last_attempt

        if self.request.method == 'POST':
            if self.username is None:
                raise DobermanImproperlyConfigured(
                    "Bad username form field, if you are using a custom field please configure: "
                    "DOBERMAN_USERNAME_FORM_FIELD via settings."
                )

            if self.response.status_code != 302:

                user_access.user_agent = self.request.META.get('HTTP_USER_AGENT', '<unknown user agent>')[:255]
                user_access.username = self.username
                user_access.failed_attempts += 1
                user_access.params_get = self.request.GET
                user_access.params_post = self.request.POST

                if user_access.failed_attempts >= self.max_failed_attempts:
                    user_access.is_locked = True

                user_access.save()

            elif self.response.status_code == 302 and not user_access.is_locked:
                user_access.is_expired = True
                user_access.save()

        return user_access

    def inspect(self):
        """
        Inspect access attempt, used for catpcha flow
        :return:
        """
        last_attempt = self.get_last_failed_access_attempt(
            ip_address=self.ip,
            captcha_enabled=True,
            captcha_passed=False,
            is_expired=False
        )

        if last_attempt is None and not self.request.user.is_authenticated():
            # create a new entry
            user_access = self._FailedAccessAttemptModel(
                ip_address=self.ip,
                username=self.username,
                captcha_enabled=True,
                captcha_passed=False,
                is_expired=False
            )
        elif last_attempt:
            user_access = last_attempt

        if self.request.method == 'POST':

            if not self.request.user.is_authenticated():

                user_access.user_agent = self.request.META.get('HTTP_USER_AGENT', '<unknown user agent>')[:255]
                user_access.username = self.username
                user_access.failed_attempts += 1
                user_access.params_get = self.request.GET
                user_access.params_post = self.request.POST

                if user_access.failed_attempts >= self.max_failed_attempts:
                    user_access.is_locked = True
                user_access.save()

            elif self.request.user.is_authenticated() and last_attempt:
                last_attempt.is_expired = True
                last_attempt.save()

    def get_lockout_response(self):
        """
        :return:
        """

        return render_to_response(
            self.template_name,
            {'user_attempts': self.last_attempt_instance,
             'lockout_time': self.block_login_seconds,
             'ip_address': self.ip
             }, context_instance=RequestContext(self.request)
        )